//////////////////////////////////////////////////////////////////////////////
// gloceanconfig.h
// -------------------
// Config module for GLOcean window decoration
// -------------------
// Copyright (c) 2004 R Paquette <bitwit@digilanti.org>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#ifndef GLOCEANCONFIG_H
#define GLOCEANCONFIG_H

#include <qobject.h>
#include <qfiledialog.h>

class Preview : public QLabel, public QFilePreview
{
    public:
        Preview( QWidget *parent=0 ) : QLabel( parent ) {}

        void previewUrl( const QUrl &u )
        {
            QString path = u.path();
            QPixmap pix( path );
            if ( pix.isNull() )
                setText( "This is not a pixmap" );
            else
                setPixmap( pix );
        }
};

class KConfig;
class ConfigDialog;

class GLOceanConfig : public QObject {
    Q_OBJECT
public:
    GLOceanConfig(KConfig* config, QWidget* parent);
    ~GLOceanConfig();
signals:
    void changed();
public slots:
    void load(KConfig*);
    void save(KConfig*);
    void defaults();
    void symbolChanged();
protected slots:
    void selectionChanged(int);
    void fileChanged(const QString &);
    void fileaChanged();
    void filebChanged();
    void getFileNameA();
    void getFileNameB();
    void aboutButtonReleased();
    
    QImage& fadeImage(QImage &img, float val = 0.5, const QColor &color = Qt::gray);

private:
    KConfig *config_;
    ConfigDialog *dialog_;
    QFileDialog* fd;
    Preview* preview;
        static const unsigned char default_bits[][8];
        static const unsigned char plastic_bits[][8];
        static const unsigned char kstep_bits[][8];
        static const unsigned char control_bits[][8];
    void setButtonBits( const unsigned char bits[][8] );
};

//// close_bits[]
//// help_bits[]
//// max_bits[]
//// min_bits[]
//// restore_bits[]
//// sticky_off_bits[]
//// sticky_on_bits[]
//// above_off_bits[]
//// above_on_bits[]
//// below_off_bits[]
//// below_on_bits[]
//// shade_off_bits[]
//// shade_on_bits[]

const unsigned char GLOceanConfig::default_bits[][8] =
        {
                { 0x00, 0x66, 0x7e, 0x3c, 0x3c, 0x7e, 0x66, 0x00 },
                { 0x7e, 0x7e, 0x60, 0x78, 0x78, 0x00, 0x18, 0x18 },
                { 0x00, 0x18, 0x3c, 0x7e, 0xff, 0xff, 0x00, 0x00 },
                { 0x00, 0x00, 0xff, 0xff, 0x7e, 0x3c, 0x18, 0x00 },
                { 0x00, 0x02, 0x06, 0x0e, 0x1e, 0x3e, 0x7e, 0x00 },
                { 0x00, 0x18, 0x18, 0x7e, 0x7e, 0x18, 0x18, 0x00 },
                { 0x00, 0x00, 0x00, 0x7e, 0x7e, 0x00, 0x00, 0x00 },
                { 0x7e, 0x18, 0x7e, 0x3c, 0x18, 0x00, 0x00, 0x00 },
                { 0x18, 0x3c, 0x7e, 0x18, 0x7e, 0x00, 0x00, 0x00 },
                { 0x00, 0x00, 0x00, 0x18, 0x3c, 0x7e, 0x18, 0x7e },
                { 0x00, 0x00, 0x00, 0x7e, 0x18, 0x7e, 0x3c, 0x18 },
                { 0xff, 0xff, 0x81, 0xa5, 0x81, 0xa5, 0x81, 0xff },
                { 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }
        };

const unsigned char GLOceanConfig::plastic_bits[][8] =
        {
                { 0x81, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x81 },
                { 0x1c, 0x36, 0x22, 0x30, 0x18, 0x08, 0x00, 0x08 },
                { 0xff, 0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0xff },
                { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff },
                { 0xfc, 0xfc, 0x84, 0x9f, 0x9f, 0xf1, 0x11, 0x1f },
                { 0x3c, 0x42, 0x81, 0x81, 0x81, 0x81, 0x42, 0x3c },
                { 0x00, 0x3c, 0x66, 0x5a, 0x5a, 0x66, 0x3c, 0x00 },
                { 0x00, 0x00, 0x18, 0x24, 0x42, 0xff, 0x00, 0x00 },
                { 0x00, 0x00, 0x18, 0x3c, 0x7e, 0xff, 0x00, 0x00 },
                { 0x00, 0x00, 0xff, 0x42, 0x24, 0x18, 0x00, 0x00 },
                { 0x00, 0x00, 0xff, 0x7e, 0x3c, 0x18, 0x00, 0x00 },
                { 0xff, 0xff, 0x81, 0xff, 0x00, 0x18, 0x3c, 0x7e },
                { 0xff, 0xff, 0x81, 0x99, 0xbd, 0xff, 0x81, 0xff }
        };

const unsigned char GLOceanConfig::kstep_bits[][8] =
        {
                { 0xc3, 0xe7, 0x7e, 0x3c, 0x3c, 0x7e, 0xe7, 0xc3 },
                { 0x18, 0x18, 0x00, 0x18, 0x18, 0x18, 0x18, 0x18 },
                { 0x18, 0x3c, 0x7e, 0x00, 0xff, 0x42, 0x24, 0x18 },
                { 0xff, 0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0xff },
                { 0xfc, 0xf8, 0xf1, 0xe3, 0xc5, 0x89, 0x11, 0x3f },
                { 0x18, 0x18, 0x18, 0xff, 0xff, 0x18, 0x18, 0x18 },
                { 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00 },
                { 0x18, 0x3c, 0x7e, 0x00, 0xff, 0xff, 0x00, 0x00 },
                { 0xff, 0xff, 0x00, 0x7e, 0x3c, 0x18, 0x00, 0x00 },
                { 0x00, 0x00, 0xff, 0xff, 0x00, 0x7e, 0x3c, 0x18 },
                { 0x00, 0x00, 0x18, 0x3c, 0x7e, 0x00, 0xff, 0xff },
                { 0xff, 0xff, 0xc3, 0xff, 0xff, 0x00, 0x00, 0x00 },
                { 0xff, 0xff, 0xc3, 0xc3, 0xc3, 0xc3, 0xff, 0xff }
        };

const unsigned char GLOceanConfig::control_bits[][8] =
        {
                { 0x81, 0xc3, 0x66, 0x3c, 0x3c, 0x66, 0xc3, 0x81 },
                { 0x3c, 0x46, 0x46, 0x40, 0x20, 0x10, 0x18, 0x18 },
                { 0x00, 0x7e, 0x42, 0x42, 0x42, 0x42, 0x7e, 0x00 },
                { 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00 },
                { 0xff, 0x89, 0x89, 0x89, 0xf9, 0x81, 0x81, 0xff },
                { 0x00, 0x3c, 0x66, 0x5a, 0x5a, 0x66, 0x3c, 0x00 },
                { 0x10, 0x30, 0x70, 0x5e, 0x7f, 0x70, 0x30, 0x10 },
                { 0xff, 0x99, 0xbd, 0xff, 0x99, 0x99, 0x81, 0xff },
                { 0xff, 0x00, 0x18, 0x3c, 0x7e, 0x18, 0x18, 0x18 },
                { 0xff, 0x81, 0x99, 0x99, 0xff, 0xbd, 0x99, 0xff },
                { 0x18, 0x18, 0x18, 0x7e, 0x3c, 0x18, 0x00, 0xff },
                { 0xff, 0x81, 0xff, 0x18, 0x3c, 0x7e, 0x18, 0xa5 },
                { 0xff, 0x81, 0xff, 0xbd, 0x99, 0x81, 0x81, 0xff }
        };


#endif // GLOCEANCONFIG_H
