/***************************************************************************
 *   Copyright (c) by Sascha Hlusiak, 2005                                 *
 *   Sascha Hlusiak <contact@saschahlusiak.de>                             *
 *                                                                         *
 *   A working !!! GLFont::class                                           *
 *   QFont Intl Support                                                    *
 *   X11 Qt Antialiasing support                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qfont.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qbitmap.h>
#include <qrect.h>
#include "glfont.h"

inline int next_p2 ( int a )
{
        int rval=1;
        while(rval<a)
                rval<<=1;
        return rval;
}


GLFont::GLFont(QFont vfont)
                :font(vfont),metrics(vfont)
{
        for (int i=0;i<65536;i++)
                textures[i]=0;
        height=metrics.height();
        list_base=0;
        init();
}

GLFont::~GLFont()
{
        glDeleteLists(list_base,65536);
        for (int i=0;i<65536;i++)
                if (textures[i])
                        glDeleteTextures(1,&textures[i]);
}

void GLFont::init()
{
        list_base=glGenLists(65536);

        for (int i=0;i<65536;i++)
                if (textures[i])
                        glDeleteTextures(1,&textures[i]);
}

/* This sets each pixel of the image to plain white,
   but the alpha channel to the R value */
QImage GLFont::adjustAlphaBuffer(QImage img)
{
        QImage res=img.copy();
        for ( int i=0; i < res.height(); i++ ) {
                uint *p = (uint*)res.scanLine( i );
                uint *end = p + res.width();
                while ( p < end ) {
                        *p = 0x00FFFFFF | ((*p & 0xFF) << 24 );
                        p++;
                }
        }
        return res;
}

void GLFont::checkText(QString text)
{
        glListBase(list_base);
        for (uint i=0;i<text.length();i++)
                if (!textures[text.at(i).unicode()]) {
                        QChar chr=text.at(i);
                        ushort unicode=text.at(i).unicode();
                        int th=next_p2(height);
                        int w=metrics.width(chr)-metrics.leftBearing(chr)-metrics.rightBearing(chr);
                        int tw=next_p2(w);

                        QPixmap pix(tw,th,-1);
                        QPainter painter;
                        QImage img;
                        QBitmap mask(tw,th);

                        glGenTextures( 1,&textures[unicode]);
                        glMatrixMode(GL_MODELVIEW);

                        pix.fill(Qt::black);
                        painter.begin(&pix);
                        painter.setFont(font);
                        painter.setPen(Qt::white);
                        painter.drawText(-metrics.leftBearing(chr),metrics.ascent(),chr,1);
                        painter.end();

                        img=pix.convertToImage();
                        img=adjustAlphaBuffer(QGLWidget::convertToGLFormat(img));
                        glBindTexture(GL_TEXTURE_2D, textures[unicode]);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
                        glTexImage2D( GL_TEXTURE_2D, 0, 4, img.width(), img.height(), 0,
                                      GL_RGBA, GL_UNSIGNED_BYTE, img.bits() );

                        glNewList(unicode,GL_COMPILE);

                        glBindTexture(GL_TEXTURE_2D,textures[unicode]);
                        glTranslatef(metrics.leftBearing(chr),0,0);
                        glBegin(GL_QUADS);
                        glTexCoord2d(0,1.0-(double)height/(double)th);
                        glVertex2f(0,height);
                        glTexCoord2d(0,1.0);
                        glVertex2f(0,0);
                        glTexCoord2d((double)w/tw,1.0);
                        glVertex2f(w,0);
                        glTexCoord2d((double)w/tw,1.0-(double)height/(double)th);
                        glVertex2f(w,height);
                        glEnd();
                        glTranslatef(metrics.width(chr)-metrics.leftBearing(chr),0,0);
                        glEndList();
                }
}

void GLFont::renderText(double x,double y,QString text,bool fade,double maxw)
{
        if (text.isNull())
                return;
        checkText(text);
        glPushAttrib(GL_LIST_BIT | GL_CURRENT_BIT  | GL_ENABLE_BIT | GL_TRANSFORM_BIT);
        glDisable(GL_LIGHTING);
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();

        float c[4];
        glGetFloatv(GL_CURRENT_COLOR,c);

        glListBase(list_base);
        glTranslatef(x,y,0);
        for (uint i=0;i<text.length();i++) {
                const double fadeout=50.0;

                if (x>maxw)
                        break;
                if (fade && x>maxw-fadeout) {
                        double alpha=(double)(x-(maxw-fadeout))/fadeout;
                        glColor4f(c[0],c[1],c[2],c[3]*(1.0-alpha));
                }

                glCallList(text.at(i).unicode());
                x+=metrics.width(text.at(i));
        }

        glPopMatrix();
        glPopAttrib();
}

void GLFont::renderText(QRect r,Qt::AlignmentFlags align,QString text,bool fade)
{
        if (text.isNull())
                return;
        double x=r.left(),y;
        y=(int)((r.top()+r.bottom()-metrics.ascent())/2);
        if (align==Qt::AlignLeft)
                x=r.left();
        if (align==Qt::AlignRight) {
                x=r.right()-metrics.width(text);
                if (x>(double)r.left())
                        fade=false;
        }
        if (align==Qt::AlignHCenter)
                x=(double)(r.left()+r.right()-metrics.width(text))/2.0;
        if (x<(double)r.left())
                x=(double)r.left();

        renderText(x,y,text,fade,r.right());
}
