//////////////////////////////////////////////////////////////////////////////
// globe.cpp
// -------------------
// GLOcean window decoration for KDE 3.3+
// -------------------
// Copyright (c) 2004 R Paquette
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////

#include <sys/time.h>
#include <qapplication.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qtimer.h>
#include <qstring.h>
#include <qwidget.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>

#include "globe.h"
#include "background.h"
#include "button.h"

enum ButtonType {
        ButtonHelp=0,
        ButtonMax,
        ButtonMin,
        ButtonClose,
        ButtonMenu,
        ButtonSticky,
        ButtonAbove,
        ButtonBelow,
        ButtonShade,
        ButtonTypeCount
};

QImage GlobeWidget::imageBack;
QImage GlobeWidget::imageButt;
bool GlobeWidget::usetitlecolor;
int GlobeWidget::alphablend;
int GlobeWidget::buttonblend;
GLfloat GlobeWidget::agray[4] = { 0.5, 0.5, 0.5, 0.5 };
GLfloat GlobeWidget::acolor[4] = { 0.5, 0.5, 0.5, 0.5 };

GlobeWidget::GlobeWidget( QWidget *parent, const char *name, QGLWidget *shared )
                : QGLWidget( parent, name, shared )
{
        alph = (float)((alphablend) * 0.01);
        bblnd = (float)((buttonblend) * 0.01);

        buttonColor[0] = buttonColor[1] = buttonColor[2] = 1.0;
        buttonColor[3] = bblnd;
        globeColor[0] = globeColor[1] = globeColor[2] = 1.0;
        globeColor[3] = alph;

        view_rotx=view_roty=view_rotz=0.0;

        lower();
        setMouseTracking(true);
        picked_=ButtonTypeCount;
        animate_ = false;
}

GlobeWidget::~GlobeWidget()
{}

void GlobeWidget::initializeGL()
{
        static GLfloat pos1[4] = { 0.0, 0.0, 0.0, 1.0 };
        static GLfloat v1[3] = { -1.0,0.0,-0.5};
        static GLfloat pos2[4] = { 0.0, 0.0, 0.0, 1.0 };
        static GLfloat v2[3] = { 1.0,0.0,-0.5};

        glFont = new GLFont(KDecoration::options()->font(false,false) );

        glEnable( GL_LIGHTING );

        glEnable( GL_LIGHT0 );

        glEnable( GL_LIGHT1 );
        glLightfv(GL_LIGHT1, GL_POSITION, pos1);
        glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, v1);
        glLighti(GL_LIGHT1, GL_SPOT_CUTOFF, 90);
        glLighti(GL_LIGHT1, GL_SPOT_EXPONENT,1);

        glEnable( GL_LIGHT2 );
        glLightfv(GL_LIGHT2, GL_POSITION, pos2);
        glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION, v2);
        glLighti(GL_LIGHT2, GL_SPOT_CUTOFF, 90);
        glLighti(GL_LIGHT2, GL_SPOT_EXPONENT,1);

        glEnable(GL_DEPTH_TEST);
        glLightfv (GL_LIGHT0, GL_AMBIENT, agray);
        glClearColor( agray[0], agray[1], agray[2], agray[3] );

        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
        glEnable(GL_BLEND);
        glShadeModel (GL_SMOOTH);
        glEnable (GL_TEXTURE_2D);
        glEnable (GL_BLEND);

        initWorld();
        initObj();

        switch(animtype_) {
        case 1:
                initGlow();
                break;
        case 2:
                initBurn();
                break;
        default:
                break;
        }
}

void GlobeWidget::initWorld()
{
        GLuint texture[1];
        QImage tex;

        if(imageBack.isNull())
                imageBack = QPixmap( background_xpm );
        tex = QGLWidget::convertToGLFormat( imageBack.smoothScale(256,256,QImage::ScaleFree) );

        glGenTextures(1, texture);
        glBindTexture(GL_TEXTURE_2D, texture[0]);
        glTexGenf(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
        glTexGenf(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, tex.width(), tex.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.bits() );

        gqWorld = glGenLists(1);
        quadric = gluNewQuadric();
        gluQuadricDrawStyle (quadric, GLU_FILL);
        gluQuadricOrientation (quadric, GLU_INSIDE);
        gluQuadricNormals (quadric, GLU_SMOOTH);
        gluQuadricTexture (quadric, true );

        glNewList(gqWorld, GL_COMPILE);
        glBindTexture(GL_TEXTURE_2D, texture[0]);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, usetitlecolor ? GL_MODULATE : GL_REPLACE);
        glEnable(GL_TEXTURE_GEN_S);
        glEnable(GL_TEXTURE_GEN_T);
        glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, globeColor );
        gluSphere (quadric, 640.0, 5, 5);
        glDisable(GL_TEXTURE_GEN_T);
        glDisable(GL_TEXTURE_GEN_S);
        glBindTexture(GL_TEXTURE_2D, 0);
        glEndList();

}

void GlobeWidget::initObj()
{
        GLuint texture[1];
        QImage tex;

        if(imageButt.isNull())
                imageButt = QPixmap( button_xpm );

        tex = convertToGLFormat( imageButt.smoothScale(64,64,QImage::ScaleFree).mirror() );

        glGenTextures(1, texture);
        glBindTexture(GL_TEXTURE_2D, texture[0]);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, tex.width(), tex.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.bits() );

        gqObj = glGenLists(1);

        creatTexObj(texture, gqObj);
}

void GlobeWidget::initGlow()
{
        GLuint texture[1];
        QImage buf, tex;
        buf=QPixmap( 64,64 );
        initImg( buf );
        tex = convertToGLFormat( buf );

        glGenTextures(1, texture);
        glBindTexture (GL_TEXTURE_2D, texture[0]);

        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, tex.width(), tex.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.bits() );
        //gluBuild2DMipmaps (GL_TEXTURE_2D, 1, 64, 64, GL_RGBA, GL_UNSIGNED_BYTE, tex.bits());

        gloObj = glGenLists(1);

        creatTexObj(texture, gloObj);
}

void GlobeWidget::begin2d()
{
        glPushMatrix();
        glMatrixMode(GL_PROJECTION);
        glPushMatrix();
        glLoadIdentity();
        glOrtho( 0, width(), height(), 0, -32, 32 );
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
}

void GlobeWidget::end2d()
{
        glMatrixMode( GL_PROJECTION );
        glPopMatrix();
        glMatrixMode( GL_MODELVIEW );
        glPopMatrix();
}

void GlobeWidget::resizeGL( int width, int height )
{
        GLfloat w,h;
        w = 2.0;
        h = w/height;
        glViewport( 0, 0, width, height );
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glFrustum( -w, w, h, -h, 1.0, 1024.0 );
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
}

void GlobeWidget::paintGL()
{
        draw();
}

void GlobeWidget::timerEvent(QTimerEvent*)
{
        updateGL();
}

void GlobeWidget::draw()
{
        view_rotx > 359.0 ? view_rotx = 0.0 : view_rotx += 1.0;
        view_roty > 359.0 ? view_roty = 0.0 : view_roty += 0.5;
        view_rotz > 355.0 ? view_rotz = 0.0 : view_rotz += 5.0;

        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
        glClear( GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT );

        if(usetitlecolor) {
                glLightfv(GL_LIGHT1, GL_DIFFUSE, backgroundColor);
                glLightfv(GL_LIGHT2, GL_DIFFUSE, blendColor);
        } else {
                glDisable(GL_LIGHT1);
                glDisable(GL_LIGHT2);
        }

        glPushMatrix();
        glColor4fv(globeColor);
        glRotatef( view_rotx, 1.0, 0.0, 0.0 );
        glRotatef( view_roty, 0.0, 1.0, 0.0 );
        glCallList(gqWorld);
        glPopMatrix();

        drawButtons();

        drawText();

}

void GlobeWidget::drawText()
{
        QString s( getTitleText().simplifyWhiteSpace() );
        int xoffset = (offLeft+(bsize_/2));
        int drawable = width()-(offLeft+offRight+bsize_);
        drawable = drawable < bsize_ ? bsize_ : drawable;    // gota have at least one space to render
        QRect r( xoffset, 0, drawable, titleHeight_);
        QRect d( xoffset+1, 1, drawable, titleHeight_);

        begin2d();
        glDisable(GL_DEPTH_TEST);
        if(hasShadow_) {
                glColor4fv(fontShadow);
                glFont->renderText ( d, titlealign_, s, hasTextFade_ );
        }
        glColor4fv(fontColor);
        glFont->renderText ( r, titlealign_, s, hasTextFade_ );
        glEnable(GL_DEPTH_TEST);
        end2d();
}

void GlobeWidget::drawGlow( GLfloat c[4] )
{
        c[0] -= buttonColor[0];
        c[1] -= buttonColor[1];
        c[2] -= buttonColor[2];

        glBlendFunc(GL_ONE,GL_DST_ALPHA);
        glPushMatrix ();
        glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, c );
        glScalef( 1.5, 1.5, 1.0 );
        glTranslatef(0.0,0.0,-1.0);
        glCallList(gloObj);
        glPopMatrix ();
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
}

void GlobeWidget::drawButtons()
{
        offLeft = 0;
        offRight = 0;

        if(!hidebutton) {
                begin2d();
                if(usetitlecolor)
                        glEnable(GL_LIGHT0);
                drawObjs( KDecoration::options()->titleButtonsLeft(), true);
                drawObjs( KDecoration::options()->titleButtonsRight(), false);
                if(usetitlecolor)
                        glDisable(GL_LIGHT0);
                end2d();
        }
}

void GlobeWidget::drawObjs( const QString &s , bool side )
{
        int a = (int)((titleHeight_-bsize_)/2);
        int b = (int)(bsize_/2);
        int w=width();
        bool type = false;
        int t=-1;

        for(uint x = 0; x < s.length();x++) {
                switch ( s[ side ? x : s.length()-x-1 ] ) {
                case 'H': // Help button
                        type = hasHelp_;
                        t=ButtonHelp;
                        break;
                case 'A': // Maximize button
                        type = hasMax_;
                        t=ButtonMax;
                        break;
                case 'I': // Minimize button
                        type = hasMin_;
                        t=ButtonMin;
                        break;
                case 'X': // Close button
                        type = hasClose_;
                        t=ButtonClose;
                        break;
                case 'M': // Menu button
                        type = false;
                        t=ButtonMenu;
                        a += bsize_;
                        break;
                case 'S': // Sticky button
                        type = true;
                        t=ButtonSticky;
                        break;
                case 'F':// Above button
                        type = true;
                        t=ButtonAbove;
                        break;
                case 'B': // Below button
                        type = true;
                        t=ButtonBelow;
                        break;
                case 'L': // Shade button
                        type = hasShade_;
                        t=ButtonShade;
                        break;
                case '_': // Spacer item
                        type = false;
                        t=ButtonTypeCount;
                        a += bsize_/2;
                        break;
                default:
                        type = false;
                        t=-1;
                        break;
                }

                if(type == true) {
                        a += bsize_;
                        glPushMatrix();
                        glTranslatef( side ? a-b : (w-a)+b, titleHeight_/2, 0.0 );
                        if( picked_ == t && animate_ ) {
                                acolor[3] = 1.0;
                        } else {
                                acolor[3] = bblnd;
                        }
                        setButtonColor(t);
                        glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, acolor );
                        glScalef( b, b, b );
                        if(animate_ && picked_ == t) {
                                switch(animtype_) {
                                case 0:
                                        glRotatef(view_rotz, 0.0, 0.0, 1.0);
                                        glCallList(gqObj);
                                        break;
                                case 1:
                                        drawGlow(acolor);
                                        break;
                                case 2:
                                        drawBurn();
                                        break;
                                default:
                                        glCallList(gqObj);
                                        break;
                                }
                        } else
                                glCallList(gqObj);

                        glPopMatrix();
                        side ? offLeft = a : offRight = a;
                }
        }
}

void GlobeWidget::setButtonColor(int t)
{
        switch(t) {
        case ButtonHelp:
                acolor[0] = buttonColor[0]+0.5;
                acolor[1] = buttonColor[1]+0.0;
                acolor[2] = buttonColor[2]+0.5;
                break;
        case ButtonMax:
                acolor[0] = buttonColor[0]+0.0;
                acolor[1] = buttonColor[1]+0.5;
                acolor[2] = buttonColor[2]+0.0;
                break;
        case ButtonMin:
                acolor[0] = buttonColor[0]+0.5;
                acolor[1] = buttonColor[1]+0.5;
                acolor[2] = buttonColor[2]+0.0;
                break;
        case ButtonClose:
                acolor[0] = buttonColor[0]+0.5;
                acolor[1] = buttonColor[1]+0.0;
                acolor[2] = buttonColor[2]+0.0;
                break;
        case ButtonSticky:
                acolor[0] = buttonColor[0]+0.0;
                acolor[1] = buttonColor[1]+0.5;
                acolor[2] = buttonColor[2]+0.5;
                break;
                //         case ButtonAbove:
                //         case ButtonBelow:
                //         case ButtonShade:
        case ButtonMenu:
                acolor[0] = buttonColor[0]+0.5;
                acolor[1] = buttonColor[1]+0.5;
                acolor[2] = buttonColor[2]+0.5;
                break;
        default:
                acolor[0] = buttonColor[0];
                acolor[1] = buttonColor[1];
                acolor[2] = buttonColor[2];
                break;
        }
}

void GlobeWidget::creatTexObj(GLuint texture[],GLint obj)
{
        glNewList(obj, GL_COMPILE);
        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, texture[0]);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
        glBegin (GL_QUADS);
        glTexCoord2f (0.0, 0.0);
        glVertex3f (-1.0, -1.0, 0);
        glTexCoord2f (1.0, 0.0);
        glVertex3f ( 1.0, -1.0, 0);
        glTexCoord2f (1.0, 1.0);
        glVertex3f ( 1.0,  1.0, 0);
        glTexCoord2f (0.0, 1.0);
        glVertex3f (-1.0,  1.0, 0);
        glEnd ();
        glBindTexture(GL_TEXTURE_2D, 0 );
        glEndList();
}

// heavily modified from colorfire.c by Andreas Gustafsson (c)1999";
#define myRandf(x) (((float)(x) * random()) / 2147483647)
#define NR_WAVES 24

GLfloat GlobeWidget::wrot[NR_WAVES];
GLfloat GlobeWidget::wtime[NR_WAVES];
GLfloat GlobeWidget::wspd[NR_WAVES];
GLfloat GlobeWidget::wmax[NR_WAVES];
GLfloat GlobeWidget::w1 = 0;
GLfloat GlobeWidget::w2 = 0;
GLfloat GlobeWidget::w3 = 0;

void GlobeWidget::initImg(QImage &buf)
{
        double x, y, r, d;
        buf.fill(0);
        r = 32;
        for (int i = 0; i < 64; i++) {
                for (int j = 0; j < 64; j++) {
                        x = abs (i - 32);
                        y = abs (j - 32);
                        d = sqrt (x * x + y * y);

                        if (d < r) {
                                d = 1 - (d / r);
                                int c = (int)(255*d);
                                buf.setPixel(i,j, qRgba(c, c, c, c) );
                        } else
                                buf.setPixel(i,j, qRgba(0, 0, 0, 0 ) );
                }
        }
}

void GlobeWidget::initBurn()
{
        GLuint texture[1];
        QImage buf, tex;
        buf=QPixmap( 64,64 );
        initImg( buf );
        tex = convertToGLFormat( buf );
        glGenTextures(1, texture);

        for (int i = 0; i < NR_WAVES; i++) {
                wtime[i] = 3.0 + myRandf (1.0);
                wrot[i] = myRandf (360);
                wspd[i] = 0.5 + myRandf (0.3);
                wmax[i] = 1.0 + myRandf (0.5);
        }

        glBindTexture (GL_TEXTURE_2D, texture[0]);
        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
        gluBuild2DMipmaps (GL_TEXTURE_2D, 1, 64, 64, GL_RGBA, GL_UNSIGNED_BYTE, tex.bits());

        burnObj = glGenLists(1);

        creatTexObj(texture, burnObj);
}

void GlobeWidget::drawWave (int nr, float fDeltaTime)
{
        float colMod = 1.0;
        glPushMatrix ();
        glScalef (wtime[nr], wtime[nr], wtime[nr]);
        if (wtime[nr] > (wmax[nr] - 1.0))
                colMod = wmax[nr] - wtime[nr];
        glRotatef (wrot[nr], 1.0, 0.0, 0.0);
        glRotatef (wrot[nr], 0.0, 1.0, 0.0);
        glRotatef (wrot[nr], 0.0, 0.0, 1.0);
        if( fDeltaTime > 0.0  )
                glCallList(burnObj);

        wtime[nr] += fDeltaTime * wspd[nr];
        if (wtime[nr] > wmax[nr]) {
                wtime[nr] = 0;
                wrot[nr] = myRandf (360);
                wspd[nr] = 0.5 + myRandf (0.3);
                wmax[nr] = 1.0 + myRandf (0.5);
        }
        glPopMatrix ();
}

void GlobeWidget::drawBurn()
{
        struct timeval now;
        static int lastTime = -1;
        float fDeltaTime;

        gettimeofday (&now, NULL);

        if (lastTime == -1) {
                fDeltaTime = 0;
        } else {
                fDeltaTime = (((now.tv_usec < lastTime) ? 1000000 : 0) + now.tv_usec - lastTime) * 0.000001;
        }
        lastTime = now.tv_usec;
        glPushMatrix ();
        glRotatef (w1, 1.0, 0, 0);
        glRotatef (w2, 0, 1.0, 0);
        glRotatef (w3, 0, 0, 1.0);
        w1 += fDeltaTime;
        if (w1 > 360)
                w1 -= 360;
        w2 += fDeltaTime;
        if (w2 > 360)
                w2 -= 360;
        w3 += fDeltaTime;
        if (w3 > 360)
                w3 -= 360;
        drawGlow(acolor);
        glBlendFunc(GL_ONE,GL_DST_ALPHA);
        for (int i = 0; i < NR_WAVES; i++)
                drawWave (i, fDeltaTime);
        drawWave (0, 0);
        glPopMatrix ();
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
}

