/***************************************************************************
 *   Copyright (C) 2004 by Rick Paquette                                   *
 *   bitwit@digilanti.org                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLOBEWIDGET_H
#define GLOBEWIDGET_H

#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <qgl.h>
#include <qwidget.h>
#include <qfontmetrics.h>
#include <kdecoration.h>
#include <qtimer.h>
#include <qbitmap.h>
#include <qpixmap.h>
#include <qimage.h>
#include "glfont.h"

#if defined(Q_CC_MSVC)
#pragma warning(disable:4305) // init: truncation from const double to float
#endif

class GLFont;

class GlobeWidget : public QGLWidget
{
public:
        GlobeWidget( QWidget *parent=0, const char *name=0, QGLWidget *shared=0 );
        virtual ~GlobeWidget();

        int timer_interval;
        int timer_id;
        GLint gqWorld;
        GLUquadricObj* quadric;
        GLint gqObj;
        GLint gloObj;
        GLint burnObj;

        GLFont *glFont;

        GLfloat alph;
        GLfloat bblnd;
        GLfloat fontColor[4];
        GLfloat fontShadow[4];
        GLfloat globeColor[4];
        GLfloat buttonColor[4];
        GLfloat backgroundColor[4];
        GLfloat blendColor[4];
        static GLfloat agray[4];
        static GLfloat acolor[4];

        static QImage imageMenu;
        static QImage imageBack;
        static QImage imageButt;
        static QColor titlecolor;
        static bool usetitlecolor;
        static int alphablend;
        static int buttonblend;

        bool hidebutton;

        static const char * const background_xpm[];
        static const char * const button_xpm[];

        static GLfloat wrot[];
        static GLfloat wtime[];
        static GLfloat wspd[];
        static GLfloat wmax[];
        static GLfloat w1;
        static GLfloat w2;
        static GLfloat w3;

        void draw();

        inline void stop()
        {
                if(timer_id) {
                        killTimer( timer_id );
                        timer_id = 0;
                }
                updateGL();
        };
        inline void start( int t )
        {
                if(timer_id)
                        stop();
                timer_id = startTimer( t );
        };
        inline void setTitleAlign( Qt::AlignmentFlags flag )
        {
                titlealign_ = flag;
        }
        inline void setButtonSize(int b)
        {
                bsize_ = b;
        };
        inline void setTitleChange( bool f )
        {
                titlechange_ = f;
        };
        inline void setTitleColors( bool flag )
        {
                usetitlecolor = flag;
        };
        inline void setHideButtons( bool flag )
        {
                hidebutton = flag;
        };
        inline void setAlphaBlend( int a )
        {
                alphablend = a;
        };
        inline void setButtonBlend( int a )
        {
                buttonblend = a;
        };
        inline void setHasHelp( bool b )
        {
                hasHelp_ = b;
        };
        inline void setHasMin( bool b )
        {
                hasMin_ = b;
        };
        inline void setHasMax( bool b )
        {
                hasMax_ = b;
        };
        inline void setHasClose( bool b )
        {
                hasClose_ = b;
        };
        inline void setHasShade( bool b )
        {
                hasShade_ = b;
        };
        inline void setHasTextFade( bool b )
        {
                hasTextFade_ = b;
        };
        inline void setHasShadow( bool b )
        {
                hasShadow_ = b;
        };
        inline void setTitleText( QString s , int h, int w)
        {
                titleText = s;
                titleHeight_ = h;
                frameSize_ = w;
        };
        inline QString getTitleText()
        {
                return titleText;
        };
        inline void setCaptionColor( QColor c )
        {
                fontColor[0] = (float)(c.red()  /16)/16;
                fontColor[1] = (float)(c.green()/15)/16;
                fontColor[2] = (float)(c.blue() /11)/16;
                fontColor[3] = alph;
                fontShadow[0]= fontColor[0] * 0.25;
                fontShadow[1]= fontColor[1] * 0.25;
                fontShadow[2]= fontColor[2] * 0.25;
                fontShadow[3]= alph * 0.25;
        };
        inline void setBackgroundColor( QColor c )
        {
                backgroundColor[0] = (float)(c.red()  /16)/16;
                backgroundColor[1] = (float)(c.green()/15)/16;
                backgroundColor[2] = (float)(c.blue() /11)/16;
                backgroundColor[3] = 1.0;
        };
        inline void setBlendColor( QColor c )
        {
                blendColor[0] = (float)(c.red()  /16)/16;
                blendColor[1] = (float)(c.green()/15)/16;
                blendColor[2] = (float)(c.blue() /11)/16;
                blendColor[3] = 1.0;
                buttonColor[3] = bblnd;
                globeColor[3] = alph;
        };

        inline void setPicked( int p )
        {
                picked_ = p;
        };
        inline void setAnimType( int t )
        {
                animtype_ = t;
        };
        inline void setAnimate( bool f )
        {
                animate_ = f;
        };
        inline void init()
        {
                initializeGL();
        };

        GLfloat view_rotx, view_roty, view_rotz;

protected:
        inline void enterEvent(QEvent *e)
        {
                QWidget::enterEvent( e );
        };
        inline void leaveEvent(QEvent *e)
        {
                QWidget::leaveEvent( e );
        };
        inline void mousePressEvent(QMouseEvent *e)
        {
                QWidget::mousePressEvent( e );
        };
        inline void mouseReleaseEvent(QMouseEvent *e)
        {
                QWidget::mouseReleaseEvent( e );
        };
        inline void mouseMoveEvent( QMouseEvent *e )
        {
                QWidget::mouseMoveEvent( e );
        };
        inline void showEvent( QShowEvent *e )
        {
                QWidget::showEvent(e);
        };
        void initializeGL();
        void resizeGL( int, int );
        void paintGL();
        void timerEvent( QTimerEvent * );
        void drawButtons();
        void drawObjs( const QString &s , bool side );
        void setButtonColor(int t);
        int offLeft;
        int offRight;
private:
        void initWorld();
        void initObj();
        void begin2d();
        void end2d();
        void drawText();
        void initGlow();
        void drawGlow(GLfloat c[4]);
        void initBurn();
        void drawWave (int nr, float fDeltaTime);
        void drawBurn();
        void initImg(QImage &buf);
        void creatTexObj(GLuint texture[],GLint obj);
        Qt::AlignmentFlags titlealign_;
        int bsize_;
        bool hasHelp_;
        bool hasMin_;
        bool hasMax_;
        bool hasClose_;
        bool hasShade_;
        bool hasShadow_;
        bool hasTextFade_;
        int picked_;
        bool animate_;
        int animtype_;
        int titleHeight_;
        bool titlechange_;
        int frameSize_;
        QString titleText;
};

#endif //GLOBEWIDGET_H
